@@ -7,13 +7,33 @@ my %ns = (
     rdf => "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
     rdfs => "http://www.w3.org/2000/01/rdf-schema#",
     foaf => "http://xmlns.com/foaf/0.1/",
+<<<<<<< scutter.pl
+=======
     scutter => "http://purl.org/net/scutter/"
+>>>>>>> 1.3
 );
 
 Class::RDF->set_db( "dbi:SQLite:scutter.db", "", "" );
 Class::RDF->define( %ns );
 Class::RDF::NS->export('rdf','rdfs','scutter','foaf');
 
+<<<<<<< scutter.pl
+my %visited;
+my @plan = shift(@ARGV) || "http://iconocla.st/misc/foaf.rdf";
+my $seeAlso = Class::RDF::Node->new( "$ns{rdfs}seeAlso" );
+
+while (1) {
+    while (my $uri = shift @plan) {
+	warn "$uri\n";
+
+	$visited{$uri} = time;
+	my $count = eval { Class::RDF->parse(uri => $uri) };
+	if ($@) {
+	    warn $@;
+	    next;
+	}
+	warn "    + $count triples added\n";
+=======
 my $start = Class::RDF->new( data => {
     rdf->type => scutter->Context,
     scutter->source => "http://iconocla.st/misc/foaf.rdf",
@@ -30,6 +50,7 @@ while (my $prospect = Class::RDF::Object->search( scutter->last_fetched => -1 ))
     if ($@) {
 	warn $@;
 	next;
+>>>>>>> 1.3
     }
     warn "+ ", scalar(@parsed), " objects added\n";
 
